from cli_common import (
    pswd_pipe,
    run_proc
)
import os
import copy

class Rnp(object):
    def __init__(self, homedir, rnp_path, rnpkey_path):
        self.__gpg = rnp_path
        self.__key_mgm_bin = rnpkey_path
        self.__common_params = ['--homedir', homedir]
        self.__password = None
        self.__userid = None
        self.__hash = None

    @property
    def key_mgm_bin(self):
        return self.__key_mgm_bin

    @property
    def rnp_bin(self):
        return self.__gpg

    @property
    def common_params(self):
        return copy.copy(self.__common_params)

    @property
    def password(self):
        return self.__password

    @password.setter
    def password(self, val):
        self.__password = val

    @property
    def userid(self):
        return self.__userid

    @userid.setter
    def userid(self, val):
        self.__userid = val

    @property
    def hash(self):
        return self.__hash

    @hash.setter
    def hash(self, val):
        self.__hash = val

    def copy(self):
        return copy.deepcopy(self)

    def _run(self, cmd, params, batch_input = None):
        retcode, _, _ = run_proc(cmd, params, batch_input)
        return retcode == 0

    def list_keys(self, secret = False):
        params = ['--list-keys', '--secret'] if secret else ['--list-keys']
        params = params + self.common_params
        return self._run(self.key_mgm_bin, params)

    def generate_key_batch(self, batch_input):
        pipe = pswd_pipe(self.__password)
        params = self.common_params
        params += ['--generate-key', '--expert']
        params += ['--pass-fd', str(pipe)]
        params += ['--userid', self.userid]
        if self.hash:
            params += ['--hash', self.hash]
        try:
            ret = self._run(self.__key_mgm_bin, params, batch_input)
        finally:
            os.close(pipe)
        return ret

    def export_key(self, output, secure = False):
        params = self.common_params
        params += ["--output", output]
        params += ["--userid", self.userid]
        params += ["--overwrite"]
        params += ["--export-key"]
        if secure:
            params += ["--secret"]
        params += [self.userid]
        return self._run(self.key_mgm_bin, params)

    def import_key(self, filename, secure = False):
        params = self.common_params
        params += ['--import-key', filename]
        return self._run(self.key_mgm_bin, params)

    def sign(self, output, input):
        pipe = pswd_pipe(self.password)
        params = self.common_params
        params += ['--pass-fd', str(pipe)]
        params += ['--userid', self.userid]
        params += ['--sign', input]
        params += ['--output', output]
        if self.hash:
            params += ['--hash', self.hash]
        try:
            ret = self._run(self.rnp_bin, params)
        finally:
            os.close(pipe)
        return ret

    def verify(self, input):
        params = self.common_params
        params += ['--verify', input]
        if self.hash:
            params += ['--hash', self.hash]
        return self._run(self.rnp_bin, params)

    def encrypt(self, recipient, output, input):
        pipe = pswd_pipe(self.password)
        params = self.common_params
        params += ['--pass-fd', str(pipe)]
        params += ['--recipient', recipient]
        params += ['--encrypt', input]
        params += ['--output', output]
        try:
            ret = self._run(self.rnp_bin, params)
        finally:
            os.close(pipe)
        return ret

    def decrypt(self, output, input):
        pipe = pswd_pipe(self.password)
        params = self.common_params
        params += ['--pass-fd', str(pipe)]
        params += ['--userid', self.userid]
        params += ['--decrypt', input]
        params += ['--output', output]
        try:
            ret = self._run(self.rnp_bin, params)
        finally:
            os.close(pipe)
        return ret



import sys
import random
import string
import logging
import os
import re
import shutil
from subprocess import Popen, PIPE

RNP_ROOT = None
WORKDIR = ''
CONSOLE_ENCODING = 'UTF-8'

class CLIError(Exception):
    def __init__(self, message, log = None):
        super(CLIError, self).__init__(message)
        self.message = message
        self.log = log
        logging.info(self.message)
        logging.debug(self.log.strip())

    def __str__(self):
        return self.message + '\n' + self.log

def set_workdir(dir):
    global WORKDIR
    WORKDIR = dir

def is_windows():
    return sys.platform.startswith('win') or sys.platform.startswith('msys')

def path_for_gpg(path):
    # GPG built for mingw/msys doesn't work with Windows paths
    if re.match(r'^[a-z]:[\\\/].*', path.lower()):
        path = '/' + path[0] + '/' + path[3:].replace('\\', '/')
    return path

def raise_err(msg, log = None):
    raise CLIError(msg, log)

def size_to_readable(num, suffix = 'B'):
    for unit in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
        if abs(num) < 1024.0:
            return "%3.1f%s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)

def list_upto(lst, count):
    return (list(lst)*(count//len(lst)+1))[:count]

def pswd_pipe(password):
    pr, pw = os.pipe()
    with os.fdopen(pw, 'w') as fw:
        fw.write(password)
        fw.write('\n')
        fw.write(password)
    os.set_inheritable(pr, True)

    if not is_windows():
        return pr
    # On Windows pipe is not inheritable so dup() is needed
    prd = os.dup(pr)
    os.close(pr)
    return prd

def random_text(path, size):
    # Generate random text, with 50% probability good-compressible
    if random.randint(0, 10) < 5:
        st = ''.join(random.choice(string.ascii_letters + string.digits + " \t\r\n-,.")
                     for _ in range(size))
    else:
        st = ''.join(random.choice("abcdef0123456789 \t\r\n-,.") for _ in range(size))
    with open(path, 'w+') as f:
        f.write(st)

def file_text(path, encoding = CONSOLE_ENCODING):
    with open(path, 'rb') as f:
        return f.read().decode(encoding).replace('\r\r', '\r')

def find_utility(name, exitifnone = True):
    path = shutil.which(name)
    if not path and exitifnone:
        logging.error('Cannot find utility {}. Exiting.'.format(name))
        sys.exit(1)

    return path

def rnp_file_path(relpath, check = True):
    global RNP_ROOT
    if not RNP_ROOT:
        pypath = os.path.dirname(__file__)
        RNP_ROOT = os.path.realpath(os.path.join(pypath, '../..'))

    fpath = os.path.realpath(os.path.join(RNP_ROOT, relpath))

    if check and not os.path.isfile(fpath):
        raise NameError('rnp: file ' + relpath + ' not found')

    return fpath

def run_proc_windows(proc, params, stdin=None):
    exe = os.path.basename(proc)
    # test special quote cases 
    params = list(map(lambda st: st.replace('"', '\\"'), params))
    # We need to escape empty parameters/ones with spaces with quotes
    params = tuple(map(lambda st: st if (st and not any(x in st for x in [' ','\r','\t'])) else '"%s"' % st, [exe] + params))
    logging.debug((proc + ' ' + ' '.join(params)).strip())
    logging.debug('Working directory: ' + os.getcwd())
    sys.stdout.flush()

    stdin_path = os.path.join(WORKDIR, 'stdin.txt')
    stdout_path = os.path.join(WORKDIR, 'stdout.txt')
    stderr_path = os.path.join(WORKDIR, 'stderr.txt')
    pass_path = os.path.join(WORKDIR, 'pass.txt')
    passfd = 0
    passfo = None
    try:
        idx = params.index('--pass-fd')
        if idx < len(params):
            passfd = int(params[idx+1])
            passfo = os.fdopen(passfd, 'r', closefd=False)
    except (ValueError, OSError):
        # Ignore if pass-fd is invalid/could not be opened
        pass
    # We may use pipes here (ensuring we use dup to inherit handles), but those have limited buffer
    # so we'll need to poll process
    if stdin:
        with open(stdin_path, "wb+") as stdinf:
            stdinf.write(stdin.encode() if isinstance(stdin, str) else stdin)
        stdin_fl = os.open(stdin_path, os.O_RDONLY | os.O_BINARY)
        stdin_no = sys.stdin.fileno()
        stdin_cp = os.dup(stdin_no)
    else:
        stdin_fl = None
        stdin_no = -1
        stdin_cp = None

    stdout_fl = os.open(stdout_path, os.O_CREAT | os.O_RDWR | os.O_BINARY)
    stdout_no = sys.stdout.fileno()
    stdout_cp = os.dup(stdout_no)
    stderr_fl = os.open(stderr_path, os.O_CREAT | os.O_RDWR | os.O_BINARY)
    stderr_no = sys.stderr.fileno()
    stderr_cp = os.dup(stderr_no)
    if passfo:
        with open(pass_path, "w+") as passf:
            passf.write(passfo.read())
        pass_fl = os.open(pass_path, os.O_RDONLY | os.O_BINARY)
        pass_cp = os.dup(passfd)

    retcode = -1
    try:
        os.dup2(stdout_fl, stdout_no)
        os.close(stdout_fl)
        os.dup2(stderr_fl, stderr_no)
        os.close(stderr_fl)
        if stdin:
            os.dup2(stdin_fl, stdin_no)
            os.close(stdin_fl)
        if passfo:
            os.dup2(pass_fl, passfd)
            os.close(pass_fl)
        retcode = os.spawnv(os.P_WAIT, proc, params)
    finally:
        os.dup2(stdout_cp, stdout_no)
        os.close(stdout_cp)
        os.dup2(stderr_cp, stderr_no)
        os.close(stderr_cp)
        if stdin:
            os.dup2(stdin_cp, stdin_no)
            os.close(stdin_cp)
        if passfo:
            os.dup2(pass_cp, passfd)
            os.close(pass_cp)
            passfo.close()
    out = file_text(stdout_path).replace('\r\n', '\n')
    err = file_text(stderr_path).replace('\r\n', '\n')
    os.unlink(stdout_path)
    os.unlink(stderr_path)
    if stdin: 
        os.unlink(stdin_path)
    if passfo: 
        os.unlink(pass_path)
    logging.debug(err.strip())
    logging.debug(out.strip())
    return (retcode, out, err)

if sys.version_info >= (3,):
    def decode_string_escape(s):
        bts = bytes(s, 'utf-8')
        result = u''
        candidate = bytearray()
        utf = bytearray()
        for b in bts:
            if b > 0x7F:
                if len(candidate) > 0:
                    result += candidate.decode('unicode-escape')
                    candidate.clear()
                utf.append(b)
            else:
                if len(utf) > 0:
                    result += utf.decode('utf-8')
                    utf.clear()
                candidate.append(b)
        if len(candidate) > 0:
            result += candidate.decode('unicode-escape')
        if len(utf) > 0:
            result += utf.decode('utf-8')
        return result
    def _decode(s):
        return s
else: # Python 2
    def decode_string_escape(s):
        return s.encode(CONSOLE_ENCODING).decode('decode_string_escape')
    def _decode(x):
        return x.decode(CONSOLE_ENCODING)

def run_proc(proc, params, stdin=None):
    # On Windows we need to use spawnv() for handle inheritance in pswd_pipe()
    if is_windows():
        return run_proc_windows(proc, params, stdin)
    paramline = u' '.join(map(_decode, params))
    logging.debug((proc + ' ' + paramline).strip())
    param_bytes = list(map(lambda x: x.encode(CONSOLE_ENCODING), params))
    process = Popen([proc] + param_bytes, stdout=PIPE, stderr=PIPE,
                    stdin=PIPE if stdin else None, close_fds=False,
                    universal_newlines=True)
    output, errout = process.communicate(stdin)
    retcode = process.poll()
    logging.debug(errout.strip())
    logging.debug(output.strip())

    return (retcode, output, errout)

def run_proc_fast(proc, params):
    with open(os.devnull, 'w') as devnull:
        proc = Popen([proc] + params, stdout=devnull, stderr=devnull)
    return proc.wait()



import copy
from cli_common import run_proc

class GnuPG(object):
    def __init__(self, homedir, gpg_path):
        self.__gpg = gpg_path
        self.__common_params = ['--homedir', homedir, '--yes']
        self.__password = None
        self.__userid = None
        self.__hash = None

    @property
    def bin(self):
        return self.__gpg

    @property
    def common_params(self):
        return copy.copy(self.__common_params)

    @property
    def password(self):
        return self.__password

    @property
    def userid(self):
        return self.__userid

    @userid.setter
    def userid(self, val):
        self.__userid = val

    @property
    def hash(self):
        return self.__hash

    @hash.setter
    def hash(self, val):
        self.__hash = val

    @password.setter
    def password(self, val):
        self.__password = val

    def copy(self):
        return copy.deepcopy(self)

    def _run(self, cmd, params, batch_input = None):
        retcode, _, _ = run_proc(cmd, params, batch_input)
        return retcode == 0

    def list_keys(self, secret = False):
        params = ['--list-secret-keys'] if secret else ['--list-keys']
        params = params + self.common_params
        return self._run(self.__gpg, params)

    def generate_key_batch(self, batch_input):
        params = ['--gen-key', '--expert', '--batch',
                  '--pinentry-mode', 'loopback'] + self.common_params
        if self.password:
            params += ['--passphrase', self.password]
        if self.hash:
            params += ['--cert-digest-algo', self.hash]
        return self._run(self.__gpg, params, batch_input)

    def export_key(self, out_filename, secret=False):
        params = ['--armor']
        if secret:
            params += ['--pinentry-mode', 'loopback', '--export-secret-key']
            params += ['--passphrase', self.password]
        else:
           params = ['--export']

        params = self.common_params + \
            params + ['-o', out_filename, self.userid]
        return self._run(self.__gpg, params)

    def import_key(self, filename, secret = False):
        params = self.common_params
        if secret:
            params += ['--trust-model', 'always']
            params += ['--batch']
            params += ['--passphrase', self.password]
        params += ['--import', filename]
        return self._run(self.__gpg, params)

    def sign(self, out, input):
        params = self.common_params
        params += ['--passphrase', self.password]
        params += ['--batch']
        params += ['--pinentry-mode', 'loopback']
        params += ['-u', self.userid]
        params += ['-o', out]
        params += ['--sign', input]
        if self.hash:
            params += ['--digest-algo', self.hash]
        return self._run(self.__gpg, params)

    def verify(self, input):
        params = self.common_params
        params += ['--verify', input]
        if self.hash:
            params += ['--digest-algo', self.hash]
        return self._run(self.__gpg, params)

    def encrypt(self, recipient, out, input):
        params = self.common_params
        params += ['--passphrase', self.password]
        params += ['-r', recipient]
        params += ['-o', out]
        # Blindely trust the key without asking
        params += ['--always-trust']
        params += ['--encrypt', input]
        return self._run(self.__gpg, params)

    def decrypt(self, out, input):
        params = self.common_params
        params += ['--passphrase', self.password]
        params += ['--pinentry-mode', 'loopback']
        params += ['-o', out]
        params += ['--decrypt', input]
        return self._run(self.__gpg, params)



#!/usr/bin/env python

import sys
import tempfile
import shutil
import inspect
import os
import logging
from timeit import default_timer as perf_timer
from argparse import ArgumentParser
from cli_common import (
    find_utility,
    run_proc,
    run_proc_fast,
    pswd_pipe,
    rnp_file_path,
    size_to_readable,
    raise_err
)

RNP = ''
RNPK = ''
GPG = ''
WORKDIR = ''
RNPDIR = ''
GPGDIR = ''
RMWORKDIR = False
SMALL_ITERATIONS = 100
LARGE_ITERATIONS = 5
LARGESIZE = 1024*1024*100
SMALLSIZE = 0
SMALLFILE = 'smalltest.txt'
LARGEFILE = 'largetest.txt'
PASSWORD = 'password'

def setup(workdir):
    # Searching for rnp and gnupg
    global RNP, GPG, RNPK, WORKDIR, RNPDIR, GPGDIR, SMALLSIZE, RMWORKDIR
    logging.basicConfig(stream=sys.stdout, format="%(message)s")
    logging.getLogger().setLevel(logging.INFO)

    RNP = rnp_file_path('src/rnp/rnp')
    RNPK = rnp_file_path('src/rnpkeys/rnpkeys')
    GPG = find_utility('gpg')
    if workdir:
        WORKDIR = workdir
    else:
        WORKDIR = tempfile.mkdtemp(prefix = 'rnpptmp')
        RMWORKDIR = True

    logging.debug('Setting up test in {} ...'.format(WORKDIR))

    # Creating working directory and populating it with test files
    RNPDIR = os.path.join(WORKDIR, '.rnp')
    GPGDIR = os.path.join(WORKDIR, '.gpg')
    os.mkdir(RNPDIR, 0o700)
    os.mkdir(GPGDIR, 0o700)

    # Generating key
    pipe = pswd_pipe(PASSWORD)
    params = ['--homedir', RNPDIR, '--pass-fd', str(pipe), '--userid', 'performance@rnp',
              '--generate-key']
    # Run key generation
    run_proc(RNPK, params)
    os.close(pipe)


    # Importing keys to GnuPG so it can build trustdb and so on
    run_proc(GPG, ['--batch', '--passphrase', '', '--homedir', GPGDIR, '--import',
                   os.path.join(RNPDIR, 'pubring.gpg'), os.path.join(RNPDIR, 'secring.gpg')])

    # Generating small file for tests
    SMALLSIZE = 3312
    st = 'lorem ipsum dol ' * (SMALLSIZE//16+1)
    with open(os.path.join(WORKDIR, SMALLFILE), 'w+') as small_file:
        small_file.write(st)

    # Generating large file for tests
    print('Generating large file of size {}'.format(size_to_readable(LARGESIZE)))

    st = '0123456789ABCDEF' * (1024//16)
    with open(os.path.join(WORKDIR, LARGEFILE), 'w') as fd:
        for i in range(0, LARGESIZE // 1024):
            fd.write(st)

def run_iterated(iterations, func, src, dst, *args):
    runtime = 0

    for i in range(0, iterations):
        tstart = perf_timer()
        func(src, dst, *args)
        runtime += perf_timer() - tstart
        os.remove(dst)

    res = runtime / iterations
    #print '{} average run time: {}'.format(func.__name__, res)
    return res

def rnp_symencrypt_file(src, dst, cipher, zlevel = 6, zalgo = 'zip', armor = False):
    params = ['--homedir', RNPDIR, '--password', PASSWORD, '--cipher', cipher,
              '-z', str(zlevel), '--' + zalgo, '-c', src, '--output', dst]
    if armor:
        params += ['--armor']
    ret = run_proc_fast(RNP, params)
    if ret != 0:
        raise_err('rnp symmetric encryption failed')

def rnp_decrypt_file(src, dst):
    ret = run_proc_fast(RNP, ['--homedir', RNPDIR, '--password', PASSWORD, '--decrypt', src,
                              '--output', dst])
    if ret != 0:
        raise_err('rnp decryption failed')

def gpg_symencrypt_file(src, dst, cipher = 'AES', zlevel = 6, zalgo = 1, armor = False):
    params = ['--homedir', GPGDIR, '-c', '-z', str(zlevel), '--s2k-count', '524288',
              '--compress-algo', str(zalgo), '--batch', '--passphrase', PASSWORD,
              '--cipher-algo', cipher, '--output', dst, src]
    if armor:
        params.insert(2, '--armor')
    ret = run_proc_fast(GPG, params)
    if ret != 0:
        raise_err('gpg symmetric encryption failed for cipher ' + cipher)

def gpg_decrypt_file(src, dst, keypass):
    ret = run_proc_fast(GPG, ['--homedir', GPGDIR, '--pinentry-mode=loopback', '--batch',
                              '--yes', '--passphrase', keypass, '--trust-model', 'always',
                              '-o', dst, '-d', src])
    if ret != 0:
        raise_err('gpg decryption failed')

def print_test_results(fsize, rnptime, gpgtime, operation):
    if not rnptime or not gpgtime:
        logging.info('{}:TEST FAILED'.format(operation))

    if fsize == SMALLSIZE:
        rnpruns = 1.0 / rnptime
        gpgruns = 1.0 / gpgtime
        runstr = '{:.2f} runs/sec vs {:.2f} runs/sec'.format(rnpruns, gpgruns)

        if rnpruns >= gpgruns:
            percents = (rnpruns - gpgruns) / gpgruns * 100
            logging.info('{:<30}: RNP is {:>3.0f}% FASTER then GnuPG ({})'.format(
                operation, percents, runstr))
        else:
            percents = (gpgruns - rnpruns) / gpgruns * 100
            logging.info('{:<30}: RNP is {:>3.0f}% SLOWER then GnuPG ({})'.format(
                operation, percents, runstr))
    else:
        rnpspeed = fsize / 1024.0 / 1024.0 / rnptime
        gpgspeed = fsize / 1024.0 / 1024.0 / gpgtime
        spdstr = '{:.2f} MB/sec vs {:.2f} MB/sec'.format(rnpspeed, gpgspeed)

        if rnpspeed >= gpgspeed:
            percents = (rnpspeed - gpgspeed) / gpgspeed * 100
            logging.info('{:<30}: RNP is {:>3.0f}% FASTER then GnuPG ({})'.format(
                operation, percents, spdstr))
        else:
            percents = (gpgspeed - rnpspeed) / gpgspeed * 100
            logging.info('{:<30}: RNP is {:>3.0f}% SLOWER then GnuPG ({})'.format(
                operation, percents, spdstr))

def get_file_params(filetype):
    if filetype == 'small':
        infile, outfile, iterations, fsize = (SMALLFILE, SMALLFILE + '.gpg',
                                              SMALL_ITERATIONS, SMALLSIZE)
    else:
        infile, outfile, iterations, fsize = (LARGEFILE, LARGEFILE + '.gpg',
                                              LARGE_ITERATIONS, LARGESIZE)

    infile = os.path.join(WORKDIR, infile)
    rnpout = os.path.join(WORKDIR, outfile + '.rnp')
    gpgout = os.path.join(WORKDIR, outfile + '.gpg')
    return (infile, rnpout, gpgout, iterations, fsize)


class Benchmark(object):
    rnphome = ['--homedir', RNPDIR]
    gpghome = ['--homedir', GPGDIR]

    def small_file_symmetric_encryption(self):
        # Running each operation iteratively for a small and large file(s), calculating the average
        # 1. Encryption
        '''
        Small file symmetric encryption
        '''
        infile, rnpout, gpgout, iterations, fsize = get_file_params('small')
        for armor in [False, True]:
            tmrnp = run_iterated(iterations, rnp_symencrypt_file, infile, rnpout,
                                 'AES128', 0, 'zip', armor)
            tmgpg = run_iterated(iterations, gpg_symencrypt_file, infile, gpgout,
                                 'AES128', 0, 1, armor)
            testname = 'ENCRYPT-SMALL-{}'.format('ARMOR' if armor else 'BINARY')
            print_test_results(fsize, tmrnp, tmgpg, testname)

    def large_file_symmetric_encryption(self):
        '''
        Large file symmetric encryption
        '''
        infile, rnpout, gpgout, iterations, fsize = get_file_params('large')
        for cipher in ['AES128', 'AES192', 'AES256', 'TWOFISH', 'BLOWFISH', 'CAST5', 'CAMELLIA128', 'CAMELLIA192', 'CAMELLIA256']:
            tmrnp = run_iterated(iterations, rnp_symencrypt_file, infile, rnpout,
                                 cipher, 0, 'zip', False)
            tmgpg = run_iterated(iterations, gpg_symencrypt_file, infile, gpgout,
                                 cipher, 0, 1, False)
            testname = 'ENCRYPT-{}-BINARY'.format(cipher)
            print_test_results(fsize, tmrnp, tmgpg, testname)

    def large_file_armored_encryption(self):
        '''
        Large file armored encryption
        '''
        infile, rnpout, gpgout, iterations, fsize = get_file_params('large')
        tmrnp = run_iterated(iterations, rnp_symencrypt_file, infile, rnpout,
                             'AES128', 0, 'zip', True)
        tmgpg = run_iterated(iterations, gpg_symencrypt_file, infile, gpgout, 'AES128', 0, 1, True)
        print_test_results(fsize, tmrnp, tmgpg, 'ENCRYPT-LARGE-ARMOR')

    def small_file_symmetric_decryption(self):
        '''
        Small file symmetric decryption
        '''
        infile, rnpout, gpgout, iterations, fsize = get_file_params('small')
        inenc = infile + '.enc'
        for armor in [False, True]:
            gpg_symencrypt_file(infile, inenc, 'AES', 0, 1, armor)
            tmrnp = run_iterated(iterations, rnp_decrypt_file, inenc, rnpout)
            tmgpg = run_iterated(iterations, gpg_decrypt_file, inenc, gpgout, PASSWORD)
            testname = 'DECRYPT-SMALL-{}'.format('ARMOR' if armor else 'BINARY')
            print_test_results(fsize, tmrnp, tmgpg, testname)
            os.remove(inenc)

    def large_file_symmetric_decryption(self):
        '''
        Large file symmetric decryption
        '''
        infile, rnpout, gpgout, iterations, fsize = get_file_params('large')
        inenc = infile + '.enc'
        for cipher in ['AES128', 'AES192', 'AES256', 'TWOFISH', 'BLOWFISH', 'CAST5',
                       'CAMELLIA128', 'CAMELLIA192', 'CAMELLIA256']:
            gpg_symencrypt_file(infile, inenc, cipher, 0, 1, False)
            tmrnp = run_iterated(iterations, rnp_decrypt_file, inenc, rnpout)
            tmgpg = run_iterated(iterations, gpg_decrypt_file, inenc, gpgout, PASSWORD)
            testname = 'DECRYPT-{}-BINARY'.format(cipher)
            print_test_results(fsize, tmrnp, tmgpg, testname)
            os.remove(inenc)

    def large_file_armored_decryption(self):
        '''
        Large file armored decryption
        '''
        infile, rnpout, gpgout, iterations, fsize = get_file_params('large')
        inenc = infile + '.enc'
        gpg_symencrypt_file(infile, inenc, 'AES128', 0, 1, True)
        tmrnp = run_iterated(iterations, rnp_decrypt_file, inenc, rnpout)
        tmgpg = run_iterated(iterations, gpg_decrypt_file, inenc, gpgout, PASSWORD)
        print_test_results(fsize, tmrnp, tmgpg, 'DECRYPT-LARGE-ARMOR')
        os.remove(inenc)

        # 3. Signing
        #print '\n#3. Signing\n'
        # 4. Verification
        #print '\n#4. Verification\n'
        # 5. Cleartext signing
        #print '\n#5. Cleartext signing and verification\n'
        # 6. Detached signature
        #print '\n#6. Detached signing and verification\n'

# Usage ./cli_perf.py [working_directory]
#
# It's better to use RAMDISK to perform tests
# in order to speed up disk reads/writes
#
# On linux:
# mkdir -p /tmp/working
# sudo mount -t tmpfs -o size=512m tmpfs /tmp/working
# ./cli_perf.py -w /tmp/working
# sudo umount /tmp/working


if __name__ == '__main__':

    # parse options
    parser = ArgumentParser(description="RNP benchmarking")
    parser.add_argument("-b", "--bench", dest="benchmarks",
                      help="Name of the comma-separated benchmarks to run", metavar="benchmarks")
    parser.add_argument("-w", "--workdir", dest="workdir",
                      help="Working directory to use", metavar="workdir")
    parser.add_argument("-l", "--list", help="Print list of available benchmarks and exit",
                        action="store_true")
    args = parser.parse_args()

    # get list of benchamrks to run
    bench_methods = [ x[0] for x in inspect.getmembers(Benchmark,
            predicate=lambda x: inspect.ismethod(x) or inspect.isfunction(x))]
    print(bench_methods)

    if args.list:
        for name in bench_methods:
            logging.info(("\t " + name))
        sys.exit(0)

    if args.benchmarks:
        bench_methods = filter(lambda x: x in args.benchmarks.split(","), bench_methods)

    # setup operations
    setup(args.workdir)

    for name in bench_methods:
        method = getattr(Benchmark, name)
        logging.info(("\n" + name + "(): " + inspect.getdoc(method)))
        method(Benchmark())

    try:
        shutil.rmtree(WORKDIR)
    except Exception:
        logging.info(("Cleanup failed"))
